@@ -17,12 +17,11 @@
 
 use datafusion::arrow::array::Array;
 use datafusion::arrow::datatypes::{DataType, IntervalUnit, TimeUnit};
-use datafusion::common::{DataFusionError, ScalarValue};
+use datafusion::common::ScalarValue;
 use datafusion::logical_expr::sqlparser::ast::NullTreatment as DFNullTreatment;
+use pyo3::exceptions::PyNotImplementedError;
 use pyo3::{exceptions::PyValueError, prelude::*};
 
-use crate::errors::py_datafusion_err;
-
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd)]
 pub struct PyScalarValue(pub ScalarValue);
 
@@ -171,9 +170,7 @@ impl DataTypeMap {
                 PythonType::Datetime,
                 SqlType::DATE,
             )),
-            DataType::Duration(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
+            DataType::Duration(_) => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
             DataType::Interval(interval_unit) => Ok(DataTypeMap::new(
                 DataType::Interval(*interval_unit),
                 PythonType::Datetime,
@@ -188,9 +185,9 @@ impl DataTypeMap {
                 PythonType::Bytes,
                 SqlType::BINARY,
             )),
-            DataType::FixedSizeBinary(_) => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", arrow_type)),
-            )),
+            DataType::FixedSizeBinary(_) => {
+                Err(PyNotImplementedError::new_err(format!("{arrow_type:?}")))
+            }
             DataType::LargeBinary => Ok(DataTypeMap::new(
                 DataType::LargeBinary,
                 PythonType::Bytes,
@@ -206,25 +203,28 @@ impl DataTypeMap {
                 PythonType::Str,
                 SqlType::VARCHAR,
             )),
-            DataType::List(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                arrow_type
-            )))),
-            DataType::FixedSizeList(_, _) => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", arrow_type)),
+            DataType::List(_) => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::FixedSizeList(_, _) => {
+                Err(PyNotImplementedError::new_err(format!("{arrow_type:?}")))
+            }
+            DataType::LargeList(_) => {
+                Err(PyNotImplementedError::new_err(format!("{arrow_type:?}")))
+            }
+            DataType::Struct(_) => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::Union(_, _) => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::Dictionary(_, _) => {
+                Err(PyNotImplementedError::new_err(format!("{arrow_type:?}")))
+            }
+            DataType::Decimal32(precision, scale) => Ok(DataTypeMap::new(
+                DataType::Decimal32(*precision, *scale),
+                PythonType::Float,
+                SqlType::DECIMAL,
+            )),
+            DataType::Decimal64(precision, scale) => Ok(DataTypeMap::new(
+                DataType::Decimal64(*precision, *scale),
+                PythonType::Float,
+                SqlType::DECIMAL,
             )),
-            DataType::LargeList(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
-            DataType::Struct(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
-            DataType::Union(_, _) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
-            DataType::Dictionary(_, _) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
             DataType::Decimal128(precision, scale) => Ok(DataTypeMap::new(
                 DataType::Decimal128(*precision, *scale),
                 PythonType::Float,
@@ -235,25 +235,16 @@ impl DataTypeMap {
                 PythonType::Float,
                 SqlType::DECIMAL,
             )),
-            DataType::Map(_, _) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
-            DataType::RunEndEncoded(_, _) => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", arrow_type)),
-            )),
-            DataType::BinaryView => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
-            DataType::Utf8View => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                arrow_type
-            )))),
-            DataType::ListView(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
-            DataType::LargeListView(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", arrow_type),
-            ))),
+            DataType::Map(_, _) => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::RunEndEncoded(_, _) => {
+                Err(PyNotImplementedError::new_err(format!("{arrow_type:?}")))
+            }
+            DataType::BinaryView => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::Utf8View => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::ListView(_) => Err(PyNotImplementedError::new_err(format!("{arrow_type:?}"))),
+            DataType::LargeListView(_) => {
+                Err(PyNotImplementedError::new_err(format!("{arrow_type:?}")))
+            }
         }
     }
 
@@ -319,33 +310,33 @@ impl DataTypeMap {
                 Ok(DataType::Interval(IntervalUnit::MonthDayNano))
             }
             ScalarValue::List(arr) => Ok(arr.data_type().to_owned()),
-            ScalarValue::Struct(_fields) => Err(py_datafusion_err(
-                DataFusionError::NotImplemented("ScalarValue::Struct".to_string()),
+            ScalarValue::Struct(_fields) => Err(PyNotImplementedError::new_err(
+                "ScalarValue::Struct".to_string(),
             )),
             ScalarValue::FixedSizeBinary(size, _) => Ok(DataType::FixedSizeBinary(*size)),
             ScalarValue::FixedSizeList(_array_ref) => {
                 // The FieldRef was removed from ScalarValue::FixedSizeList in
                 // https://github.com/apache/arrow-datafusion/pull/8221, so we can no
                 // longer convert back to a DataType here
-                Err(py_datafusion_err(DataFusionError::NotImplemented(
+                Err(PyNotImplementedError::new_err(
                     "ScalarValue::FixedSizeList".to_string(),
-                )))
+                ))
             }
-            ScalarValue::LargeList(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
+            ScalarValue::LargeList(_) => Err(PyNotImplementedError::new_err(
                 "ScalarValue::LargeList".to_string(),
-            ))),
+            )),
             ScalarValue::DurationSecond(_) => Ok(DataType::Duration(TimeUnit::Second)),
             ScalarValue::DurationMillisecond(_) => Ok(DataType::Duration(TimeUnit::Millisecond)),
             ScalarValue::DurationMicrosecond(_) => Ok(DataType::Duration(TimeUnit::Microsecond)),
             ScalarValue::DurationNanosecond(_) => Ok(DataType::Duration(TimeUnit::Nanosecond)),
-            ScalarValue::Union(_, _, _) => Err(py_datafusion_err(DataFusionError::NotImplemented(
+            ScalarValue::Union(_, _, _) => Err(PyNotImplementedError::new_err(
                 "ScalarValue::LargeList".to_string(),
-            ))),
+            )),
             ScalarValue::Utf8View(_) => Ok(DataType::Utf8View),
             ScalarValue::BinaryView(_) => Ok(DataType::BinaryView),
-            ScalarValue::Map(_) => Err(py_datafusion_err(DataFusionError::NotImplemented(
+            ScalarValue::Map(_) => Err(PyNotImplementedError::new_err(
                 "ScalarValue::Map".to_string(),
-            ))),
+            )),
         }
     }
 }
@@ -379,8 +370,7 @@ impl DataTypeMap {
             "double" => Ok(DataType::Float64),
             "byte_array" => Ok(DataType::Utf8),
             _ => Err(PyValueError::new_err(format!(
-                "Unable to determine Arrow Data Type from Parquet String type: {:?}",
-                parquet_str_type
+                "Unable to determine Arrow Data Type from Parquet String type: {parquet_str_type:?}"
             ))),
         };
         DataTypeMap::map_from_arrow_type(&arrow_dtype?)
@@ -403,14 +393,8 @@ impl DataTypeMap {
     #[pyo3(name = "sql")]
     pub fn py_map_from_sql_type(sql_type: &SqlType) -> PyResult<DataTypeMap> {
         match sql_type {
-            SqlType::ANY => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::ARRAY => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::ANY => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::ARRAY => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::BIGINT => Ok(DataTypeMap::new(
                 DataType::Int64,
                 PythonType::Int,
@@ -431,13 +415,8 @@ impl DataTypeMap {
                 PythonType::Int,
                 SqlType::CHAR,
             )),
-            SqlType::COLUMN_LIST => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::CURSOR => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::COLUMN_LIST => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::CURSOR => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::DATE => Ok(DataTypeMap::new(
                 DataType::Date64,
                 PythonType::Datetime,
@@ -448,139 +427,88 @@ impl DataTypeMap {
                 PythonType::Float,
                 SqlType::DECIMAL,
             )),
-            SqlType::DISTINCT => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::DISTINCT => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::DOUBLE => Ok(DataTypeMap::new(
                 DataType::Decimal256(1, 1),
                 PythonType::Float,
                 SqlType::DOUBLE,
             )),
-            SqlType::DYNAMIC_STAR => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
+            SqlType::DYNAMIC_STAR => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::FLOAT => Ok(DataTypeMap::new(
                 DataType::Decimal128(1, 1),
                 PythonType::Float,
                 SqlType::FLOAT,
             )),
-            SqlType::GEOMETRY => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::GEOMETRY => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::INTEGER => Ok(DataTypeMap::new(
                 DataType::Int8,
                 PythonType::Int,
                 SqlType::INTEGER,
             )),
-            SqlType::INTERVAL => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::INTERVAL_DAY => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_DAY_HOUR => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_DAY_MINUTE => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::INTERVAL_DAY_SECOND => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::INTERVAL_HOUR => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_HOUR_MINUTE => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::INTERVAL_HOUR_SECOND => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::INTERVAL_MINUTE => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_MINUTE_SECOND => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::INTERVAL_MONTH => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_SECOND => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_YEAR => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::INTERVAL_YEAR_MONTH => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::MAP => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::MULTISET => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::INTERVAL => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::INTERVAL_DAY => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::INTERVAL_DAY_HOUR => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_DAY_MINUTE => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_DAY_SECOND => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_HOUR => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::INTERVAL_HOUR_MINUTE => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_HOUR_SECOND => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_MINUTE => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_MINUTE_SECOND => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_MONTH => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::INTERVAL_SECOND => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::INTERVAL_YEAR => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::INTERVAL_YEAR_MONTH => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::MAP => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::MULTISET => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::NULL => Ok(DataTypeMap::new(
                 DataType::Null,
                 PythonType::None,
                 SqlType::NULL,
             )),
-            SqlType::OTHER => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::REAL => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::ROW => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::SARG => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::OTHER => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::REAL => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::ROW => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::SARG => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::SMALLINT => Ok(DataTypeMap::new(
                 DataType::Int16,
                 PythonType::Int,
                 SqlType::SMALLINT,
             )),
-            SqlType::STRUCTURED => Err(py_datafusion_err(DataFusionError::NotImplemented(
-                format!("{:?}", sql_type),
-            ))),
-            SqlType::SYMBOL => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::TIME => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::TIME_WITH_LOCAL_TIME_ZONE => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
-            SqlType::TIMESTAMP => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
-            SqlType::TIMESTAMP_WITH_LOCAL_TIME_ZONE => Err(py_datafusion_err(
-                DataFusionError::NotImplemented(format!("{:?}", sql_type)),
-            )),
+            SqlType::STRUCTURED => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::SYMBOL => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::TIME => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::TIME_WITH_LOCAL_TIME_ZONE => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
+            SqlType::TIMESTAMP => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
+            SqlType::TIMESTAMP_WITH_LOCAL_TIME_ZONE => {
+                Err(PyNotImplementedError::new_err(format!("{sql_type:?}")))
+            }
             SqlType::TINYINT => Ok(DataTypeMap::new(
                 DataType::Int8,
                 PythonType::Int,
                 SqlType::TINYINT,
             )),
-            SqlType::UNKNOWN => Err(py_datafusion_err(DataFusionError::NotImplemented(format!(
-                "{:?}",
-                sql_type
-            )))),
+            SqlType::UNKNOWN => Err(PyNotImplementedError::new_err(format!("{sql_type:?}"))),
             SqlType::VARBINARY => Ok(DataTypeMap::new(
                 DataType::LargeBinary,
                 PythonType::Bytes,
@@ -595,7 +523,7 @@ impl DataTypeMap {
     }
 
     /// Unfortunately PyO3 does not allow for us to expose the DataType as an enum since
-    /// we cannot directly annotae the Enum instance of dependency code. Therefore, here
+    /// we cannot directly annotate the Enum instance of dependency code. Therefore, here
     /// we provide an enum to mimic it.
     #[pyo3(name = "friendly_arrow_type_name")]
     pub fn friendly_arrow_type_name(&self) -> PyResult<&str> {
@@ -631,6 +559,8 @@ impl DataTypeMap {
             DataType::Struct(_) => "Struct",
             DataType::Union(_, _) => "Union",
             DataType::Dictionary(_, _) => "Dictionary",
+            DataType::Decimal32(_, _) => "Decimal32",
+            DataType::Decimal64(_, _) => "Decimal64",
             DataType::Decimal128(_, _) => "Decimal128",
             DataType::Decimal256(_, _) => "Decimal256",
             DataType::Map(_, _) => "Map",
@@ -682,8 +612,7 @@ impl PyDataType {
             "datetime64" => Ok(DataType::Date64),
             "object" => Ok(DataType::Utf8),
             _ => Err(PyValueError::new_err(format!(
-                "Unable to determine Arrow Data Type from Arrow String type: {:?}",
-                arrow_str_type
+                "Unable to determine Arrow Data Type from Arrow String type: {arrow_str_type:?}"
             ))),
         };
         Ok(PyDataType {
