@@ -75,6 +75,6 @@ impl PyAggregateFunction {
 
     /// Get a String representation of this column
     fn __repr__(&self) -> String {
-        format!("{}", self)
+        format!("{self}")
     }
 }
