@@ -65,7 +65,7 @@ impl PyEmptyRelation {
 
     /// Get a String representation of this column
     fn __repr__(&self) -> String {
-        format!("{}", self)
+        format!("{self}")
     }
 
     fn __name__(&self) -> PyResult<String> {
