@@ -85,7 +85,7 @@ impl PyProjection {
     }
 
     fn __repr__(&self) -> PyResult<String> {
-        Ok(format!("Projection({})", self))
+        Ok(format!("Projection({self})"))
     }
 
     fn __name__(&self) -> PyResult<String> {
