@@ -66,7 +66,7 @@ impl PyUnion {
     }
 
     fn __repr__(&self) -> PyResult<String> {
-        Ok(format!("Union({})", self))
+        Ok(format!("Union({self})"))
     }
 
     fn __name__(&self) -> PyResult<String> {
