@@ -201,8 +201,7 @@ impl PyLogicalPlan {
         let proto_plan =
             datafusion_proto::protobuf::LogicalPlanNode::decode(bytes).map_err(|e| {
                 PyRuntimeError::new_err(format!(
-                    "Unable to decode logical node from serialized bytes: {}",
-                    e
+                    "Unable to decode logical node from serialized bytes: {e}"
                 ))
             })?;
 
